/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * Portions created by Devexperts are Copyright (C) 2007-2012 Devexperts.
 */

package com.devexperts.evolution.utils.configencoder;

import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;

/**
 * manages aliasing for ConfigEncoder
 */
class AliasTable {
    public static final int NO_ALIAS = -1;
    int nextAliasNumber = 0;
    HashMap<String, Entry> names = new HashMap<String, Entry>();
    ArrayList<Entry> aliases = new ArrayList<Entry>();

    public void addName(String packageName) {
        Entry objectName = names.get(packageName);
        if (objectName != null) {
            objectName.setPackageNameRepeats(true);
        } else {
            names.put(packageName, new Entry(packageName));
        }
    }

    public int getAlias(String packageName) {
        Entry entry = names.get(packageName);
        if (entry != null && entry.isPackageNameRepeats()) {
            if (!entry.isAliasAssigned()) {
                entry.setNumber(nextAliasNumber++);
                aliases.add(entry);
            }
            return entry.getNumber();
        }
        return NO_ALIAS;
    }

    public void clear() {
        names.clear();
    }

    public void write(Writer out) throws IOException {
        if (aliases.isEmpty()) {
            return;
        }

        out.append('%');
        boolean notFirst = false;
        for (Entry alias : aliases) {
            if (notFirst) {
                out.append(',');
            }
            out.append(Integer.toString(alias.getNumber())).append(':').append(alias.getPackageName());
            notFirst = true;
        }
        out.append('%');
    }

    public String getClassAlias(String className) {
        String packageName = Utils.extractPackageName(className);
        int alias = getAlias(packageName);
        if (alias != NO_ALIAS) {
            return "%" + alias + "." + Utils.extractClassName(className);
        }
        return className;
    }

    /**
     * Holds mapped package name with additional info for aliasing
     */
    private static class Entry {
        /**
         * encoded package name
         */
        private String packageName;
        /**
         * assigned aliasing number
         */
        private int number = NO_ALIAS;
        private boolean packageNameRepeats = false;

        public Entry(String packageName) {
            this.packageName = packageName;
        }

        public String getPackageName() {
            return packageName;
        }

        public int getNumber() {
            return number;
        }

        public void setNumber(int number) {
            this.number = number;
        }

        public boolean isPackageNameRepeats() {
            return packageNameRepeats;
        }

        public void setPackageNameRepeats(boolean packageNameRepeats) {
            this.packageNameRepeats = packageNameRepeats;
        }

        public boolean isAliasAssigned() {
            return number != NO_ALIAS;
        }
    }
}
