/*
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * Portions created by Devexperts are Copyright (C) 2007-2012 Devexperts.
 */

package com.devexperts.evolution.utils.configencoder;

/**
 * @author Andrey Belomutskiy 8/19/11
 */
public class ClassResolver {
    public Object classForName(String name, ClassLoader loader) throws ClassNotFoundException {
        ClassLoader loader1;
        if (loader != null) {
            loader1 = loader;
        } else {
            loader1 = Thread.currentThread().getContextClassLoader();
        }
        return classForName2(name, loader1);
    }

    public static Class classForName2(String name, ClassLoader cl)
        throws ClassNotFoundException
    {
        if (cl == null) {
            throw new IllegalArgumentException("ClassLoader cannot be null");
        }

        Class primitiveType = typeNameToPrimitiveClass(name);
        if (primitiveType != null) {
            return primitiveType;
        }
        return Class.forName(name, false, cl);
    }

    public static Class typeNameToPrimitiveClass(String typeName) {
        typeName = typeName.intern();
        // todo: with 'intern()', simpler (=faster) '==' should be OK instead of 'equals()'
        if (typeName.equals("boolean")) {
            return boolean.class;
        }
        if (typeName.equals("byte")) {
            return byte.class;
        }
        if (typeName.equals("char")) {
            return char.class;
        }
        if (typeName.equals("short")) {
            return short.class;
        }
        if (typeName.equals("int")) {
            return int.class;
        }
        if (typeName.equals("long")) {
            return long.class;
        }
        if (typeName.equals("float")) {
            return float.class;
        }
        if (typeName.equals("double")) {
            return double.class;
        }
        if (typeName.equals("void")) {
            return void.class;
        }
        return null;
    }
}
